function An = Oro_nasal_tract_area_mat( Ant, Lnt, Avpo, Lvel )
% Create the area and length functions matrix for the nasal tract,
% according to the VPO function. Use a linear interpolation at the
% velopharyngeal section.
%
% Input arguments :
%
% Ant : Area function
% Lnt : length function
% Avpo : VPO function
% Lvel : length of the velum (default = 3 cm)
%
% Output argument :
%
% A : Area matrix
%
% Benjamin Elie March 2014

if nargin < 6; Lvel = 3e-2; end;

Nt = length(Avpo);
% Ant and Lnt must be column vectors
Ant = Ant(:);
An = repmat(Ant,[1,Nt]);
xnt = cumsum(Lnt);
ind = find(xnt<=Lvel, 1, 'last');
Avkm1 = -1;

for k = 1:Nt   
    if Avpo(k) ~= Avkm1    
        An(1:ind,k) = interp1(xnt([1 ind]), [Avpo(k) An(ind,k)], xnt(1:ind)); 
    else        
        An(1:ind,k) = An(1:ind,k-1);            
    end    
    Avkm1 = Avpo(k);    
end

end
 
