clearvars
close all
clc

set(0,'defaulttextinterpreter','latex')
set(0,'defaultAxesFontSize',20)

%%%%% initialization of parameters
ndim = 3;
fso = 2e4; %%% Output sampling frequency (in Hz)
fsim = fso*ndim; % simulation frequency (in Hz)
vmin = 1e-11; % Minimal value to avoid division by 0
agmax = 0.4*1e-4; % Maximal glottal opening

ii = dir('input_data/*.mat');
%%%% Choose vowel (among 'a','i','o','u','y')
vow = 'i';

% load input data as vectors of configurations at each time step
load(fullfile('input_data',[vow,'_data.mat']),'aot', 'lot','avpo', 'kn', 'psub', 'tvec','ago','fo');

%%%%% Uncomment if you want the output fundamental frequency to be 10 kHz (to gain speed)
% aot = aot(:,1:2:end); % aot is the vector of area function
% lot = lot(:,1:2:end); % lot is the vector of length function
% avpo = avpo(1:2:end); % avpo is the vectior of velum opening
% psub = psub(1:2:end); % psub is the subglottal pressure at each time step
% tvec = tvec(1:2:end); % tvec is the time points of the simulation
% ago = ago(1:2:end); % ago is the abduction degree, taken between 0 (for closed glottis) and 1 (for fully open glottis)
% fo = fo(1:2:end); % fo is the vector of pitch values at each time step

aot(aot<=vmin) = vmin; % if area = 0 => set to the minimal value
ag = agmax*ago; % set glottal opening as area
ag(ag<=vmin) = vmin; % if glottal opening area = 0 => set to the minimal value
nt = length(tvec); % number of simulation time points

%%%% Gender of the speaker ('M' for male, 'F' for female)
Gend = 'M';

%%%% Definition of the area function of the waveguide network
k = 1;
af(k).A = aot; af(k).l = lot;
Coup = struct('child',[],'parents',[],'twin',[],'anabranch',[],'rad',1,'Nat',{[]});

nasal_tract = 0; %%%% NASAL TRACT? %%%%%%
ch = 0; %%%%%% GLOTTAL CHINK? %%%%%%%%%%
if nasal_tract % If connection to the nasal tract
    run(fullfile('library','Nasal_tract_Serrurier.m')); % get nasal tract area function from Serrurier PhD thesis
    kn = round(median(kn)); % Need a int value
    Lvel = 3e-2; % Length of the velum (m)
    ant = Oro_nasal_tract_area_mat( Ant, Lnt, avpo, Lvel ); % modify the first areas depending on the velum opening
    lnt = repmat(Lnt,[1,nt]); clear Lnt Lvel 
    Antleft(1:4) = ant(1:4);
    ant(ant<=vmin) = vmin; lnt(lnt<=vmin) = vmin;
    af(2).A = ant; af(2).l = lnt; % New waveguide
    Coup = struct('child',{[ 2; kn ], [] },'parents',{[],[1;kn]},'twin',{[],[]},'anabranch',cell(1,2),'rad',{1,1},'Nat',{[],[]});
end
    
if ch % If Glottal chink    
    lch = ago.^2; % adimensional length of the chink opening
    lch(lch<=vmin) = vmin; % if length = 0 => set to the minimal value
    lch(lch>=1) = 1-vmin; %  if length = 1 => set to the maximal value (1-vmin)
    inparam.lch = lch;
    nwave = length(af); % New waveguide
    Coup(nwave+1) = struct('child',{[]},'parents',{[1;1]},'twin',{[]},'anabranch',{[]},'rad',{0},'Nat',{'Chink'});
end

%%%%%%%%% INITIALIZATION
inparam.Psub = psub;
inparam.Ag0 = zeros(size(tvec));
inparam.AgP = ones(size(tvec));
inparam.fo = fo;
run(fullfile('library','constinit.m')); % Get constant parameters
run(fullfile('library','Vocal_folds_init.m')); % Initialization of the VF parameters
if ch; af(nwave+1).A = reshape(ag(:),size(tvec)); af(nwave+1).l = xvec(end)*ones(size(tvec)); end
disp('Start simulation')
const.rmlt = 4;%800;
const.namp = 1e-6;
tic
[ p_o, ug_o, aco_o, acdc_o, h_o, k_o, m_o, y_o, w_o ] = esmfsynthesis(tvec, af, Coup, inparam, vfparam, const);
toc

%%% Save matfile
save(fullfile('output_data','matfiles',[vow,'_data.mat']),'p_o', 'ug_o', 'aco_o', 'acdc_o', 'h_o', 'k_o', 'm_o', 'y_o', 'w_o','tvec','af','Coup','inparam','vfparam','const');
p_o(isnan(p_o)) = 0;
% profile viewer
disp('Done')
pr = p_o;%
% pr = timedewarping( pr, max(af(1).l), 1024, fsim);
if ndim > 1; pr = resample(pr,1,ndim,30); end
%%% write audio file
audiowrite(fullfile('output_data','audio',[vow,'_data.wav']),pr/max(abs(pr))*0.9999, fso);


%%%% FOR DISPLAY
%%% spectrogram
nwin = 64;
novlp = nwin-4;
nfft = 2^nextpow2(fso);
[ Sxx, fsp, tsp ] = spectrogram(pr,hann(nwin), novlp, nfft, fso);
Sxx = Sxx./max(Sxx(:));

close all
figure;
h(1)=subplot(3,1,1);  plot(tvec, p_o/max(abs(p_o))); title(['Audio waveform of simulated /',vow,'/'])
set(gca,'ticklabelinterpreter','latex')
h(2)=subplot(3,1,2); plot(tvec, min(af(1).A)); hold on; title('Miniminal cross-section area')
set(gca,'ticklabelinterpreter','latex')
ylabel('Area (m\textsuperscript{2})')
if ch
    plot(tvec, af(end).A, 'r')
    leg1=legend('Supraglottal tract','Glottal opening','Location','Best');
    set(leg1,'interpreter','latex')
end
h(3)=subplot(313); plot(tvec, inparam.fo);title('Fundamental frequency')
set(gca,'ticklabelinterpreter','latex')
ylabel('$F_0$ (Hz)')
xlabel('Time (s)')
% h(4) = subplot(414); plot(tvec, inparam.fo)
linkaxes(h,'x');
xlim([0 tvec(end)])
set(gcf,'units','normalized','position',[0 0 1 1])

figure;
h2(1) = subplot(3,1,1:2); 
imagesc(tsp, fsp, db(Sxx)); axis xy; caxis([-50 0]); colormap(1-gray);
set(gca,'ticklabelinterpreter','latex');
title(['Spectrogram of simulated /',vow,'/'])
ylabel('Frequency (Hz)')
h2(2) = subplot(313);  plot(tvec, p_o/max(abs(p_o))); title('Audio waveform')
set(gca,'ticklabelinterpreter','latex')
xlabel('Time (s)')
linkaxes(h2,'x');
set(gcf,'units','normalized','position',[0 0 1 1])

xlim([0 tvec(end)])